
#include "fxttypes.h"
#include "perm/permq.h"
#include "ds/bitarray.h"

#include "fxtio.h"

void
print_cycles(const ulong *f, ulong n, bitarray *tb/*=0*/)
// Print cycle form of the permutation in f[].
// Examples (first permutations of 4 elements in lex order):
//       array form      cycle form
//   0:  [ 0 1 2 3 ]    (0) (1) (2) (3)
//   1:  [ 0 1 3 2 ]    (0) (1) (2, 3)
//   2:  [ 0 2 1 3 ]    (0) (1, 2) (3)
//   3:  [ 0 2 3 1 ]    (0) (1, 2, 3)
//   4:  [ 0 3 1 2 ]    (0) (1, 3, 2)
//   5:  [ 0 3 2 1 ]    (0) (1, 3) (2)
//   6:  [ 1 0 2 3 ]    (0, 1) (2) (3)
//   7:  [ 1 0 3 2 ]    (0, 1) (2, 3)
//   8:  [ 1 2 0 3 ]    (0, 1, 2) (3)
{
    bitarray *b = tb;
    if ( tb==0 )  b = new bitarray(n);
    b->clear_all();

    ulong np = 4*n - 4;  // for padding with n <= 10

    for (ulong k=0; k<n; ++k)
    {
        if ( b->test(k) )  continue; // already processed

        cout << "(";  np-=1;
        ulong i = k;  // next in cycle
//        const char *cm = "";
        do
        {
            cout << i;
            if ( f[i]!=k )  { cout << ", ";  np-=2; }
//            cout << cm << i;
//            cm = ", ";
            b->set(i);
        }
        while ( (i=f[i]) != k );  // until we meet cycle leader again
        cout << ") ";  np-=2;
    }

    if ( n<=10 )  for (  ; np; --np)  cout << " ";

    if ( tb==0 )  delete b;
}
// -------------------------

//ulong
//print_cycles(const ulong *f, ulong n, bitarray *bp/*=0*/)
//// Print the cycles of the permutation.
//// Return number of fixed points.
//{
//    bitarray *tp = bp;
//    if ( 0==bp )  tp = new bitarray(n);  // tags
//    tp->clear_all();
//
//    ulong ct = 0;  // # of fixed points
//    for (ulong k=0; k<n; ++k)
//    {
//        if ( tp->test_clear(k) )  continue;  // already processed
//        tp->set(k);
//
//        // follow a cycle:
//        ulong i = k;
//        ulong g = f[i];  // next index
//        if ( g==i )  // fixed point ?
//        {
//            ++ct;
//            continue;
//        }
//
//        cout << " ( " << i;
//        while ( 0==(tp->test_set(g)) )
//        {
////            cout << " <-- " << setw(3) << g;
//            cout << ", " << g;
//            g = f[g];
//        }
//        cout << " )";
//    }
//
//    if ( 0==bp )  delete tp;
//
//    return  ct;
//}
//// -------------------------


ulong
print_cycle(const ulong *f, ulong e0/*=0*/)
// Print the cycle that contains e0.
// Return length of the cycle.
{
    ulong e = e0;
    ulong ct = 0;
    cout << "(";
    do
    {
        ++ct;
        cout << e;
        e=f[e];
        if ( e!=e0 )  cout << ", ";
    }
    while ( e!=e0 );
    cout << ")";
    return ct;
}
// -------------------------

